package net.vtstar.zhongtong.avi.gateway.painting.job;

import lombok.extern.slf4j.Slf4j;
import net.vtstar.scada.base.equipmgt.domain.Enum.EquipStatusEnum;
import net.vtstar.scada.base.gather.domain.EquipmentData;
import net.vtstar.scada.base.gather.job.GatherJob;
import net.vtstar.zhongtong.avi.gateway.painting.domain.PaintingRobotMachine;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 涂装机器人数据采集JOB
 */
@Slf4j
@Component
public class PaintingRobotMachineJob extends GatherJob {

    private static final String NAME = "PAINTING_ROBOT";

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {

        super.gatherByModbus(NAME, PaintingRobotMachine.class);
    }

    @Override
    public boolean handleObject(EquipmentData equipmentData) {
        PaintingRobotMachine data = (PaintingRobotMachine) equipmentData;
        String state = data.getStatus();
        if (state == null || "0".equals(state)) {
            return false;
        }
        equipmentData.setEquipStatus(EquipStatusEnum.IDLE);
        if(null != data.getRunState() && data.getRunState() == 1){
            equipmentData.setEquipStatus(EquipStatusEnum.RUN);
        }
        if(null != data.getErrorState() && data.getErrorState() == 1){
            equipmentData.setEquipStatus(EquipStatusEnum.ALARM);
        }
        if(null != data.getRunStatusR11() && data.getRunStatusR11() == 1){
            equipmentData.setEquipStatus(EquipStatusEnum.RUN);
        }
        if(null != data.getRunStatusR21() && data.getRunStatusR21() == 1){
            equipmentData.setEquipStatus(EquipStatusEnum.RUN);
        }
        return true;
    }

    @Override
    public Object handleValue(String fieldName, Object value) {
        Integer valueInt = Integer.valueOf(value.toString());
        return (valueInt & 0x01);
    }

    @Override
    protected void postHandle(EquipmentData equipmentData) {
        ((PaintingRobotMachine)equipmentData).setCreateTime(new Date());
    }
}
