package net.vtstar.zhongtong.avi.utils;

import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.SmbFile;
import jcifs.smb.SmbFileInputStream;
import jcifs.smb.SmbFileOutputStream;
import lombok.extern.slf4j.Slf4j;

import java.io.*;

/**
 * 远程共享文件访问（上传、下发）
 *
 * @Author: xiadongwei
 * @Date: 2019/10/11 10:40
 */
@Slf4j
public class SmbFileUtil {

    public static void main(String[] args) throws Exception {
        //smb://xxx:xxx@192.168.2.188/testIndex/
        //xxx:xxx是共享机器的用户名密码
        String url = "smb://10.100.172.223/NCPrg/";
        NtlmPasswordAuthentication auth = new NtlmPasswordAuthentication(null, "PA8000", null);
        SmbFile file = new SmbFile(url, auth);
        if (file.exists()) {
            SmbFile[] files = file.listFiles();
            for (SmbFile f : files) {
                System.out.println(f.getName());
            }
        }
//        smbGet("smb://10.100.172.223/NCPrg/新建文本文档.txt", "D:/111111.txt");
        smbPut("smb://10.100.172.223/NCPrg/", "D:/yield.xls", auth);

    }

    //    从共享目录下载文件
    public static void smbGet(String remoteUrl, String localDir) {
        InputStream in = null;
        OutputStream out = null;
        try {
            SmbFile remoteFile = new SmbFile(remoteUrl);
            if (!remoteFile.exists()) {
                System.out.println("共享文件不存在");
                return;
            }
            File file = new File(localDir);
            if (!file.exists()) {
                file.mkdirs();
            }
            String fileName = remoteFile.getName();
            File localFile = new File(localDir + File.separator + fileName);
            in = new BufferedInputStream(new SmbFileInputStream(remoteFile));
            out = new BufferedOutputStream(new FileOutputStream(localFile));
            byte[] buffer = new byte[1024];
            while (in.read(buffer) != -1) {
                out.write(buffer);
                buffer = new byte[1024];
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (in != null) {
                    in.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    //向共享目录上传文件
    public static Boolean smbPut(String remoteUrl, String localFilePath, NtlmPasswordAuthentication auth) {
        log.error("共享文件路径:{}", remoteUrl);
        InputStream in = null;
        OutputStream out = null;
        try {
            File localFile = new File(localFilePath);
            String fileName = localFile.getName();
            SmbFile remoteFile;
            if (null == auth){
                remoteFile = new SmbFile(remoteUrl + fileName);
            }else {
                remoteFile = new SmbFile(remoteUrl + fileName, auth);
            }

            in = new BufferedInputStream(new FileInputStream(localFile));
            out = new BufferedOutputStream(new SmbFileOutputStream(remoteFile));
            byte[] buffer = new byte[1024];
            while (in.read(buffer) != -1) {
                out.write(buffer);
                buffer = new byte[1024];
            }
        } catch (Exception e) {
            log.error("下发文件失败: {}", e.getMessage() + e.getLocalizedMessage());
            e.printStackTrace();
            return false;
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (in != null) {
                    in.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return true;
    }

}
